@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:100;src:local("Roboto Thin "),local("Roboto-Thin"),url(/static/media/roboto-latin-100.c2aa4ab1.woff2) format("woff2"),url(/static/media/roboto-latin-100.a45108d3.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:100;src:local("Roboto Thin italic"),local("Roboto-Thinitalic"),url(/static/media/roboto-latin-100italic.7f839a86.woff2) format("woff2"),url(/static/media/roboto-latin-100italic.451d4e55.woff) format("woff")}@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:300;src:local("Roboto Light "),local("Roboto-Light"),url(/static/media/roboto-latin-300.37a7069d.woff2) format("woff2"),url(/static/media/roboto-latin-300.865f928c.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:300;src:local("Roboto Light italic"),local("Roboto-Lightitalic"),url(/static/media/roboto-latin-300italic.c64e7e35.woff2) format("woff2"),url(/static/media/roboto-latin-300italic.bd5b7a13.woff) format("woff")}@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:400;src:local("Roboto Regular "),local("Roboto-Regular"),url(/static/media/roboto-latin-400.176f8f5b.woff2) format("woff2"),url(/static/media/roboto-latin-400.49ae34d4.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:400;src:local("Roboto Regular italic"),local("Roboto-Regularitalic"),url(/static/media/roboto-latin-400italic.d022bc70.woff2) format("woff2"),url(/static/media/roboto-latin-400italic.b1d9d990.woff) format("woff")}@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:500;src:local("Roboto Medium "),local("Roboto-Medium"),url(/static/media/roboto-latin-500.f5b74d7f.woff2) format("woff2"),url(/static/media/roboto-latin-500.cea99d3e.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:500;src:local("Roboto Medium italic"),local("Roboto-Mediumitalic"),url(/static/media/roboto-latin-500italic.0d8bb5b3.woff2) format("woff2"),url(/static/media/roboto-latin-500italic.18d00f73.woff) format("woff")}@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:700;src:local("Roboto Bold "),local("Roboto-Bold"),url(/static/media/roboto-latin-700.c18ee39f.woff2) format("woff2"),url(/static/media/roboto-latin-700.2267169e.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:700;src:local("Roboto Bold italic"),local("Roboto-Bolditalic"),url(/static/media/roboto-latin-700italic.7d8125ff.woff2) format("woff2"),url(/static/media/roboto-latin-700italic.9360531f.woff) format("woff")}@font-face{font-family:"Roboto";font-style:normal;font-display:swap;font-weight:900;src:local("Roboto Black "),local("Roboto-Black"),url(/static/media/roboto-latin-900.870c8c14.woff2) format("woff2"),url(/static/media/roboto-latin-900.bac8362e.woff) format("woff")}@font-face{font-family:"Roboto";font-style:italic;font-display:swap;font-weight:900;src:local("Roboto Black italic"),local("Roboto-Blackitalic"),url(/static/media/roboto-latin-900italic.cb5ad999.woff2) format("woff2"),url(/static/media/roboto-latin-900italic.c20d916c.woff) format("woff")}
/*# sourceMappingURL=2.1ca50200.chunk.css.map */