(this.webpackJsonpezmote=this.webpackJsonpezmote||[]).push([[3],{416:function(t,e,n){"use strict";function o(t){return o="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(t){return typeof t}:function(t){return t&&"function"===typeof Symbol&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},o(t)}function r(){}n.r(e);var u={getItem:r,setItem:r,removeItem:r};function f(t){var e="".concat(t,"Storage");return function(t){if("object"!==("undefined"===typeof self?"undefined":o(self))||!(t in self))return!1;try{var e=self[t],n="redux-persist ".concat(t," test");e.setItem(n,"test"),e.getItem(n),e.removeItem(n)}catch(r){return!1}return!0}(e)?self[e]:u}e.default=function(t){var e=f(t);return{getItem:function(t){return new Promise((function(n,o){n(e.getItem(t))}))},setItem:function(t,n){return new Promise((function(o,r){o(e.setItem(t,n))}))},removeItem:function(t){return new Promise((function(n,o){n(e.removeItem(t))}))}}}("local")}}]);
//# sourceMappingURL=3.6b0298ca.chunk.js.map